import datetime
import glob
import json
import logging
import os
import subprocess

import requests
from django.conf import settings
from django.contrib import messages
from django.contrib.auth.forms import AuthenticationForm
from django.core.files.storage import default_storage
from django.db.models import Q
from django.http import HttpResponseForbidden, HttpResponseNotAllowed, HttpResponseRedirect, JsonResponse
from django.urls import reverse
from django.utils.translation import ugettext_lazy as _
from django.views.generic.base import TemplateView
from emissions.models import Nonstop
from nonstop.models import SomaLogLine, Track

from panikdb.context_processors import internal_ip
from panikdb.service_messages.models import Message

from .models import PigeEmailRequest

logger = logging.getLogger('panikdb')


def is_internal(request):
    return internal_ip(request).get('internal_ip')


def is_regie_computer(request):
    return internal_ip(request).get('current_ip') in settings.REGIE_IPS


class RegieHome(TemplateView):
    template_name = 'regie-home.html'

    def get_context_data(self, **kwargs):
        context = super().get_context_data(**kwargs)
        context['login_form'] = AuthenticationForm()

        context['service_message'] = Message.objects.all().first()

        if settings.PIGE_DOWNLOAD_BASE_URL:
            context['has_pige'] = True
            context['pige_minimum_date'] = datetime.datetime.now() - datetime.timedelta(days=100)
            context['pige_hours'] = ['%02d' % x for x in range(24)]
            context['pige_minutes'] = ['%02d' % x for x in range(0, 60, 15)]

        if getattr(settings, 'NONSTOP_ON_AIR_SWITCH_URL', None) and getattr(self, 'request', None):
            context['is_regie_computer'] = is_regie_computer(self.request)
            context['has_switch'] = True

        return context


regie_home = RegieHome.as_view()


def pige_post(request, *args, **kwargs):
    if request.method != 'POST':
        return HttpResponseNotAllowed(['POST'])
    # studio2-20160515-13h45-14h00m05.flac
    name = '%(src)s-%(date)s-%(start_hour)sh%(start_min)s-%(end_hour)sh%(end_min)s' % {
        'src': request.POST['src'],
        'date': request.POST['date'].replace('-', ''),
        'start_hour': request.POST['start_hour'],
        'start_min': request.POST['start_min'],
        'end_hour': request.POST['end_hour'],
        'end_min': request.POST['end_min'],
    }
    if is_internal(request):
        name = name + '.wav'
    else:
        name = name + '.ogg'
    if 'sendmail' in request.POST:
        email = request.POST['email']
        PigeEmailRequest.objects.create(email=email, download_spec=name)
        messages.info(
            request,
            _(
                'An email with a download link will soon be sent to %s, '
                'please be patient, it may take up to 20 minutes to arrive.'
            )
            % email,
        )
        return HttpResponseRedirect(reverse('regie-home'))
    else:  # direct download
        return HttpResponseRedirect(settings.PIGE_DOWNLOAD_BASE_URL + '/' + name)


def json_tracks(tracks):
    return JsonResponse(
        {
            'data': [
                {
                    'id': track.id,
                    'title': track.title,
                    'artist': track.artist.name if track.artist_id else None,
                    'instru': track.instru,
                    'language': track.language,
                    'duration': '%d:%02d'
                    % (track.duration.total_seconds() / 60, track.duration.total_seconds() % 60),
                }
                for track in tracks
            ]
        }
    )


def regie_tracks(request):
    now = datetime.datetime.now()
    current_zone = None
    for nonstop in Nonstop.objects.all():
        if (nonstop.start < nonstop.end and (now.time() >= nonstop.start and now.time() < nonstop.end)) or (
            nonstop.start > nonstop.end and (now.time() >= nonstop.start or now.time() < nonstop.end)
        ):
            current_zone = nonstop
            break

    tracks = Track.objects.filter(duration__isnull=False)
    tracks = tracks.filter(duration__gte=datetime.timedelta(minutes=2, seconds=30))
    tracks = tracks.filter(duration__lt=datetime.timedelta(minutes=4, seconds=30))
    if current_zone:
        tracks = tracks.filter(nonstop_zones__in=[current_zone.id])
    tracks = tracks.exclude(language__isnull=True).exclude(language='')
    tracks = tracks.order_by('?')
    return json_tracks(tracks[:5])


def regie_tracks_search(request):
    q = request.GET['q']
    if q.startswith('https://'):
        ytdl_path = default_storage.path('ytdl')
        if not os.path.exists(ytdl_path):
            os.mkdir(ytdl_path)
        # make a first run with just metadata, this is to obtain the filename
        # and use a file that was previously downloaded.
        proc = subprocess.run(
            [
                'youtube-dl',
                '--extract-audio',
                '--no-playlist',
                '--no-mtime',
                '--dump-json',
                '--audio-format',
                'opus',
                '-o',
                ytdl_path + '/%(id)s.%(ext)s',
                q,
            ],
            capture_output=True,
        )
        output = json.loads(proc.stdout)
        filename = output['_filename']
        # audio conversion may be done by youtube-dl and filename would still
        # contain the original filename, look for files with same basename,
        # regardless of the extension.
        base_filename = os.path.splitext(filename)[0]
        try:
            real_filename = [
                x
                for x in glob.glob(base_filename + '*')
                if not x.endswith('.part') and not x.endswith('.json')
            ][0]
        except IndexError:
            # not yet downloaded, store json for async downloading with the ytdl
            # management command
            ready = False
            real_filename = os.path.splitext(filename)[0] + '.json'
            output['_url'] = q
            with open(real_filename, 'w') as fd:
                json.dump(output, fp=fd, indent=2)
        else:
            ready = True
        url = os.path.join(settings.MEDIA_URL, 'ytdl', real_filename[len(ytdl_path) + 1 :])
        title = output['title']
        duration = '%d:%02d' % (int(output['duration']) / 60, int(output['duration']) % 60)
        good_filename = '%s%s' % (title, os.path.splitext(real_filename)[-1])
        return JsonResponse(
            {
                'data': [
                    {
                        'title': title,
                        'duration': duration,
                        'url': url,
                        'filename': good_filename,
                        'ready': ready,
                    }
                ]
            }
        )

    tracks = Track.objects.filter(duration__isnull=False).filter(
        Q(title__icontains=q.lower()) | Q(artist__name__icontains=q.lower())
    )
    tracks = tracks.order_by('?')
    return json_tracks(tracks[:5])


def playing(request):
    latest_played = SomaLogLine.objects.select_related('track', 'track__artist').latest('play_timestamp')
    duration = latest_played.track.duration
    elapsed = datetime.datetime.now() - latest_played.play_timestamp
    remaining = duration - elapsed
    if remaining.total_seconds() < 2:
        return JsonResponse({})
    elif remaining.total_seconds() < 0:
        remaining = datetime.timedelta(seconds=0)
    response = {}
    if latest_played.track.artist:
        response['artist'] = latest_played.track.artist.name
    response['title'] = latest_played.track.title
    response['duration'] = duration.seconds
    response['elapsed'] = int(elapsed.total_seconds())
    return JsonResponse(response)


def switch(request, *args, **kwargs):
    if not is_regie_computer(request):
        return HttpResponseForbidden()
    logger.info('regie switch to %s', request.GET['selection'])
    requests.post(settings.NONSTOP_ON_AIR_SWITCH_URL, data={'s': request.GET['selection']})
    return JsonResponse({'err': 0})
